#ifndef AUDIOFILE_H
#define AUDIOFILE_H
#include "AudioFile.h"
#endif 
/* fehlercodes */
#define DATAMISSING -1 /* wird zurueckgeliefert, wenn keine daten zum dekodieren vorliegen */
#define MAXAMPL_ERROR -2 /* wird zurueckgeliefert, wenn sich in einem signal nicht die maximalamplitude bestimmen laesst */
#define SIGNALLENGTH_ERROR -3 /* wird zurueckgeliefert, wenn das zu dekodierende signal zu kurz ist */
/* laengendefinitionen der vektoren */
#define START_LENGTH 8192 /* die startlaenge ist die laenge, die angibt wie lang das ausgeschnittene stueck aus den orifinaldaten sein soll */
#define HALF_LENGTH 4096 /* die halbe startlaenge */
#define DOUBLE_LENGTH 16384 /* die doppelte startlaenge */
#define MAX_HAMMING 1024 /* anzahl der maximal moeglichen hammingfenster */

/*
 * die klasse AudioEdit ist in der lage die audiovorverarbeitung zu bewltigen.
 * sie dekodiert den vorgefundenen datenvektor (aus einer au-datei) in sieben
 * schritten: semiinvertierung, ausschneiden eines 8K langen vektors, 
 * exponentielle skalierung, fast fourier transformation (fft), 
 * teilung des (aus der fft) gewonnenen vektors, berechnung der amplituden,
 * reduzierung des datenvektors.
 */ 
class AudioEdit
{
 public:

  /* konstruktor ohne parameter */ 
  AudioEdit();
  
  /* 
   * weiterer konstruktor
   * parameter: af - ein AudioFile-objekt, welches eine au-datei bereits ausgelesen hat.
   */
  AudioEdit(AudioFile *af);
  
  /* destruktor */
  ~AudioEdit();

  /* 
   * gibt informationen ueber die inhalte (vektoren aus der vorverarbeitung!) dieses objektes auf stdout aus.
   * fuer debug-zwecke gedacht
   * parameter: flag - 0 gibt den originalvektor aus
   *                   1 gibt den ausgeschnittenen vektor aus
   *                   2 gibt den nach der fft aus
   *                   3 gibt den vektor mit den berechneten amplituden aus
   *                   4 gibt den ergebnisvektor aus
   * rueckgabe: keine
   */
  void text(int flag);

  /*
   * setter zum manuellem setzen eines datenvektors, der vorverarbeitet werden soll.
   * parameter: data - der eingabevektor (double-werte)
   * rueckgabe: keine
   */
  void setDataOriginal(double* data);

  /*
   * setter zum manuellem setzen der laenge eines eingabevektors.
   * parameter:  length - die laenge des vektors
   * rueckgabe: keine
   */
  void setDataLength(long length);

  /*
   * setter zum setzen der redundanz, die bentigt wird,
   * um den eingabevektor des neuralen netzes zu berechnen.
   * parameter: red - die redundanz ( 0 < red < 1)
   * rueckgabe: keine
   */
  void setRedundance(double red);

  /*
   * setter zum setzen des skalierungsfaktors, der bentigt wird,
   * um den eingabevektor des neuralen netzes zu berechnen.
   * parameter: sf - der skalierungsfaktor
   * rueckgabe: keine
   */
  void setScaleFactor(double sf);

  /*
   * setter zum setzen der anzahl der hammingfenster, die bentigt wird,
   * um den eingabevektor des neuralen netzes zu berechnen.
   * parameter: anz - die anzahl der fenster
   * rueckgabe: keine
   */
  void setHammingFenster(int anz);

  /*
   * decode fuer die dekodierung der audiodaten in sieben schritten durch.
   * parameter: keine
   * rueckgabe: bei fehler: DATAMISSING, MAXAMPL_ERROR oder SIGNALLENGTH_ERROR, sonst 0
   */
  int decode();

  /*
   * getter, der den originaldatenvektor zurueckliefert.
   * ist nur fuer meik!
   * parameter: keine
   * rueckgabe: originaldatenvektor (schon semiinvertiert)
   */
  double* getDataOrg();

  /*
   * getter, der die anzahl der hammingfenster zurueckgibt.
   * parameter: keine
   * rueckgabe: anzahl der hammingfenster
   */
  int getHammingFenster();

  /*
   * getter, der den datenvektor zurueckliefert nach dem ausschneiden zurueckliefert.
   * parameter: keine
   * rueckgabe: ausgeschnittener datenvektor
   */
  double* getDataStartLength();

  /*
   * getter, der den datenvektor zurueckliefert nach der fft zurueckliefert.
   * parameter: keine
   * rueckgabe: datenvektor nach der fft
   */
  double* getDataFFT();

  /*
   * getter, der den datenvektor zurueckliefert, der das endergebnis der audiovorverarbeitung darstellt.
   * parameter: keine
   * rueckgabe: eingabevektor fuer das neurale netz
   */
  double* getData();

 private: 
  
  /*
   * die init-methode allokiert speicher, setzt flags und variablen auf default-werte
   * parameter: keine
   * rueckgabe: keine
   */
  void init();

  /*  false, solange bis AudioEdit originaldaten zum verarbeiten hat */
  bool orgdataex;
  
  /* dataLength beinhaltet die laenge des originaldatenvektors */
  long dataLength;

  /* hier ist der originaldatenvektor wiederzufinden */
  double* dataOriginal;

  /* hier ist der datenvektor nach dem ausschneiden wiederzufinden */
  double* dataStartLength;

  /* hier ist der datenvektor nach der exponentiellen skalierung zu finden */
  double* dataExpo;

  /* hier ist der datenvektor nach der fft wiederzufinden */
  double* dataFFT;

  /* hier ist der datenvektor nach berechnung der amplituden wiederzufinden */
  double* dataAmpl;

  /* hier ist der ergebnisvektor wiederzufinden */
  double* data;

  /* in redundance wird die vom benutzer angegebene redundanz zur berechnung des eingabevektors fr das neurale netz abgelegt */
  double redundance;

  /* in scaleFactor wird der vom benutzer angegebene skalierungsfaktor zur berechnung des eingabevektors fr das neurale netz abgelegt */
  double scaleFactor;

  /* in hammingFenster wird die vom benutzer angegebene anzahl der hammingfenster zur berechnung des eingabevektors fr das neurale netz abgelegt */
  int hammingFenster;

  /* in audiofile wird das AudioFile-objekt abgelegt, welches die zu dekodierenden daten aus der au-datei ausgelesen hat */
  AudioFile *audiofile;

  /*
   * die methode semiinv fuehrt die semiinvertierung durch.
   * parameter: keine - aber dataOriginal wird umgewandelt
   * rueckgabe: keine
   */
  void semiinv();

  /*
   * die methode cutStartLength fuehrt das Ausschneiden der laenge START_LENGTH durch.
   * parameter: keine - dataOriginal wird verwendet, aber nicht veraendert
   * rueckgabe: bei fehler: MAXAMPL_ERROR oder SIGNALLENGTH_ERROR, sonst 0
   */
  int cutStartLength();

  /*
   * die methode expoScale fuehrt die exponentielle skalierung durch.
   * parameter: keine - dataStartLenght wird verwendet
   * rueckgabe: keine
   */
  void expoScale();

  /*
   * die funktion runFFT startet die fast fourier transformation.
     * paraemter: keine - die informationen werden von dataStartLength bezogen
   * rueckgabe: keine - das ergebnis der transformation wird in dataFFT abgelegt
   */
  void runFFT();

  /* teilt den aus der fft gewonnenen vektor.
   * parameter: keine - die informationen werden von dataFFT bezogen
   * rueckgabe: keine - dataFFT wird mit dem ergebnis ueberschrieben
   */
  void divVector();
  
  /* computeAmpl berechnet die amplitudenwerte, welche die energiedichte darstellen.
   * parameter: keine - die informationen werden von dataFFT bezogen
   * rueckgabe: keine - dataAmpl wird mit dem ergebnis beliefert
   */
  void computeAmpl();

  /* reduceData reduziert den datenvektor so, dass er als eingabevektor fuer das neurale netz tauglich ist.
   * parameter: keine - die informationen werden von dataAmpl bezogen
   * rueckgabe: keine - data wird mit dem ergebnis beliefert
   */
  void reduceData();
};
